#ifndef software_channelSAN_H_
#define software_channelSAN_H_

#include "Cpp/BaseClasses/EmptyGroup.h"
#include "Cpp/BaseClasses/PreselectGroup.h"
#include "Cpp/BaseClasses/PostselectGroup.h"
#include "Cpp/BaseClasses/state/StructStateVariable.h"
#include "Cpp/BaseClasses/state/ArrayStateVariable.h"
#include "Cpp/BaseClasses/SAN/SANModel.h" 
#include "Cpp/BaseClasses/SAN/Place.h"
#include "Cpp/BaseClasses/SAN/ExtendedPlace.h"
extern double ch1_init_pfd;
extern double ch2_init_pfd;
extern short demand_types_ch1;
extern short demand_types_ch2;
extern double upgrade_duration;
extern short patching_enabled;
extern double patchingInterval;
extern short demand_types_max;
extern double patching_channel_pfd_reduction;
extern double patching_common_pfd_reduction;
extern int patching_model;
extern double attack_success_prob_reduction;
extern double demand_default_rate;
extern double delay_sync_attack;
extern short time_sync_enabled;
extern short isPatchingDelayed;
extern double compromise_detection_prob;
extern UserDistributions* TheDistribution;

void MemoryError();

#ifndef _FR_description_header_
#define _FR_description_header_
typedef struct FR_description_state {short ID; double pfd_size; };

class FR_description: public StructStateVariable<FR_description_state> {
  public:
  ExtendedPlace<short>* ID;
  ExtendedPlace<double>* pfd_size;
  FR_description(char* name):
    StructStateVariable<FR_description_state>(name, "") {
    ID = new ExtendedPlace<short>("ID", name);
    fields.push_back(field(Short, ID));
    pfd_size = new ExtendedPlace<double>("pfd_size", name);
    fields.push_back(field(Double, pfd_size));
    }
  FR_description(char* name, char* fullname):
    StructStateVariable<FR_description_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ID = new ExtendedPlace<short>("ID", varname);
    fields.push_back(field(Short, ID));
    pfd_size = new ExtendedPlace<double>("pfd_size", varname);
    fields.push_back(field(Double, pfd_size));
    }
  FR_description(char* name, FR_description_state initialValue): 
    StructStateVariable<FR_description_state>(name, "") {
    ID = new ExtendedPlace<short>("ID", name, initialValue.ID);
    fields.push_back(field(Short, ID));
    pfd_size = new ExtendedPlace<double>("pfd_size", name, initialValue.pfd_size);
    fields.push_back(field(Double, pfd_size));
    }
  FR_description(char* name, char* fullname, FR_description_state initialValue): 
    StructStateVariable<FR_description_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ID = new ExtendedPlace<short>("ID", varname, initialValue.ID);
    fields.push_back(field(Short, ID));
    sprintf(varname, "%s.%s", fullname, name);
    pfd_size = new ExtendedPlace<double>("pfd_size", varname, initialValue.pfd_size);
    fields.push_back(field(Double, pfd_size));
    }
  ~FR_description() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i).field;
  }
};
#endif
#ifndef _FR_Array_header_
#define _FR_Array_header_

typedef FR_description_state FR_Array_state;
class FR_Array: public ArrayStateVariable<FR_description > {
  public:
  FR_Array(char* name, char* fullname):
    ArrayStateVariable<FR_description >(name, fullname, Struct ,100) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new FR_description(varname, fqname));
    }
  }

  FR_Array(char* name):
    ArrayStateVariable<FR_description >(name, "", Struct ,100) {
    char varname[12];
    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new FR_description(varname, name));
    }
  }

  FR_Array(char* name, char* fullname, FR_description_state & initialValue):
    ArrayStateVariable<FR_description >(name, fullname, Struct ,100) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new FR_description(varname, fqname, initialValue));
    }
  }

  FR_Array(char* name, FR_description_state & initialValue):
    ArrayStateVariable<FR_description >(name, "", Struct ,100) {
    char varname[12];
    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new FR_description(varname, name, initialValue));
    }
  }

  FR_Array(char* name, char* fullname, FR_description_state * initialValue):
    ArrayStateVariable<FR_description >(name, fullname, Struct ,100) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new FR_description(varname, fqname, initialValue[i]));
    }
  }

  FR_Array(char* name, FR_description_state * initialValue):
    ArrayStateVariable<FR_description >(name, "", Struct ,100) {
    char varname[12];
    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new FR_description(varname, name, initialValue[i]));
    }
  }
  ~FR_Array() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif
#ifndef _FR_collection_header_
#define _FR_collection_header_

typedef short FR_collection_state;
class FR_collection: public ArrayStateVariable<ExtendedPlace<short> > {
  public:
  FR_collection(char* name, char* fullname):
    ArrayStateVariable<ExtendedPlace<short> >(name, fullname, Short ,10) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<10;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, fqname));
    }
  }

  FR_collection(char* name):
    ArrayStateVariable<ExtendedPlace<short> >(name, "", Short ,10) {
    char varname[12];
    for (int i=0;i<10;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, name));
    }
  }

  FR_collection(char* name, char* fullname, short & initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, fullname, Short ,10) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<10;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, fqname, initialValue));
    }
  }

  FR_collection(char* name, short & initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, "", Short ,10) {
    char varname[12];
    for (int i=0;i<10;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, name, initialValue));
    }
  }

  FR_collection(char* name, char* fullname, short * initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, fullname, Short ,10) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<10;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, fqname, initialValue[i]));
    }
  }

  FR_collection(char* name, short * initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, "", Short ,10) {
    char varname[12];
    for (int i=0;i<10;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, name, initialValue[i]));
    }
  }
  ~FR_collection() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif
#ifndef _maliciousDemand_header_
#define _maliciousDemand_header_
typedef struct maliciousDemand_state {short ID; FR_collection_state FR_set; double AttackRate; double successProb; };

class maliciousDemand: public StructStateVariable<maliciousDemand_state> {
  public:
  ExtendedPlace<short>* ID;
  FR_collection* FR_set;
  ExtendedPlace<double>* AttackRate;
  ExtendedPlace<double>* successProb;
  maliciousDemand(char* name):
    StructStateVariable<maliciousDemand_state>(name, "") {
    ID = new ExtendedPlace<short>("ID", name);
    fields.push_back(field(Short, ID));
    FR_set = new FR_collection("FR_set", name);
    fields.push_back(field(Array, FR_set));
    AttackRate = new ExtendedPlace<double>("AttackRate", name);
    fields.push_back(field(Double, AttackRate));
    successProb = new ExtendedPlace<double>("successProb", name);
    fields.push_back(field(Double, successProb));
    }
  maliciousDemand(char* name, char* fullname):
    StructStateVariable<maliciousDemand_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ID = new ExtendedPlace<short>("ID", varname);
    fields.push_back(field(Short, ID));
    FR_set = new FR_collection("FR_set", varname);
    fields.push_back(field(Array, FR_set));
    AttackRate = new ExtendedPlace<double>("AttackRate", varname);
    fields.push_back(field(Double, AttackRate));
    successProb = new ExtendedPlace<double>("successProb", varname);
    fields.push_back(field(Double, successProb));
    }
  maliciousDemand(char* name, maliciousDemand_state initialValue): 
    StructStateVariable<maliciousDemand_state>(name, "") {
    ID = new ExtendedPlace<short>("ID", name, initialValue.ID);
    fields.push_back(field(Short, ID));
    FR_set = new FR_collection("FR_set", name, initialValue.FR_set);
    fields.push_back(field(Array, FR_set));
    AttackRate = new ExtendedPlace<double>("AttackRate", name, initialValue.AttackRate);
    fields.push_back(field(Double, AttackRate));
    successProb = new ExtendedPlace<double>("successProb", name, initialValue.successProb);
    fields.push_back(field(Double, successProb));
    }
  maliciousDemand(char* name, char* fullname, maliciousDemand_state initialValue): 
    StructStateVariable<maliciousDemand_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ID = new ExtendedPlace<short>("ID", varname, initialValue.ID);
    fields.push_back(field(Short, ID));
    sprintf(varname, "%s.%s", fullname, name);
    FR_set = new FR_collection("FR_set", varname, initialValue.FR_set);
    fields.push_back(field(Array, FR_set));
    sprintf(varname, "%s.%s", fullname, name);
    AttackRate = new ExtendedPlace<double>("AttackRate", varname, initialValue.AttackRate);
    fields.push_back(field(Double, AttackRate));
    sprintf(varname, "%s.%s", fullname, name);
    successProb = new ExtendedPlace<double>("successProb", varname, initialValue.successProb);
    fields.push_back(field(Double, successProb));
    }
  ~maliciousDemand() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i).field;
  }
};
#endif
#ifndef _channelAttack_header_
#define _channelAttack_header_
typedef struct channelAttack_state {short ChanID; maliciousDemand_state demandDescription; };

class channelAttack: public StructStateVariable<channelAttack_state> {
  public:
  ExtendedPlace<short>* ChanID;
  maliciousDemand* demandDescription;
  channelAttack(char* name):
    StructStateVariable<channelAttack_state>(name, "") {
    ChanID = new ExtendedPlace<short>("ChanID", name);
    fields.push_back(field(Short, ChanID));
    demandDescription = new maliciousDemand("demandDescription", name);
    fields.push_back(field(Struct, demandDescription));
    }
  channelAttack(char* name, char* fullname):
    StructStateVariable<channelAttack_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ChanID = new ExtendedPlace<short>("ChanID", varname);
    fields.push_back(field(Short, ChanID));
    demandDescription = new maliciousDemand("demandDescription", varname);
    fields.push_back(field(Struct, demandDescription));
    }
  channelAttack(char* name, channelAttack_state initialValue): 
    StructStateVariable<channelAttack_state>(name, "") {
    ChanID = new ExtendedPlace<short>("ChanID", name, initialValue.ChanID);
    fields.push_back(field(Short, ChanID));
    demandDescription = new maliciousDemand("demandDescription", name, initialValue.demandDescription);
    fields.push_back(field(Struct, demandDescription));
    }
  channelAttack(char* name, char* fullname, channelAttack_state initialValue): 
    StructStateVariable<channelAttack_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ChanID = new ExtendedPlace<short>("ChanID", varname, initialValue.ChanID);
    fields.push_back(field(Short, ChanID));
    sprintf(varname, "%s.%s", fullname, name);
    demandDescription = new maliciousDemand("demandDescription", varname, initialValue.demandDescription);
    fields.push_back(field(Struct, demandDescription));
    }
  ~channelAttack() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i).field;
  }
};
#endif
#ifndef _MaliciousDemandsCollection_header_
#define _MaliciousDemandsCollection_header_

typedef maliciousDemand_state MaliciousDemandsCollection_state[20];
class MaliciousDemandsCollection: public ArrayStateVariable<maliciousDemand > {
  public:
  MaliciousDemandsCollection(char* name, char* fullname):
    ArrayStateVariable<maliciousDemand >(name, fullname, Struct ,20) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<20;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new maliciousDemand(varname, fqname));
    }
  }

  MaliciousDemandsCollection(char* name):
    ArrayStateVariable<maliciousDemand >(name, "", Struct ,20) {
    char varname[12];
    for (int i=0;i<20;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new maliciousDemand(varname, name));
    }
  }

  MaliciousDemandsCollection(char* name, char* fullname, maliciousDemand_state & initialValue):
    ArrayStateVariable<maliciousDemand >(name, fullname, Struct ,20) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<20;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new maliciousDemand(varname, fqname, initialValue));
    }
  }

  MaliciousDemandsCollection(char* name, maliciousDemand_state & initialValue):
    ArrayStateVariable<maliciousDemand >(name, "", Struct ,20) {
    char varname[12];
    for (int i=0;i<20;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new maliciousDemand(varname, name, initialValue));
    }
  }

  MaliciousDemandsCollection(char* name, char* fullname, maliciousDemand_state * initialValue):
    ArrayStateVariable<maliciousDemand >(name, fullname, Struct ,20) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<20;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new maliciousDemand(varname, fqname, initialValue[i]));
    }
  }

  MaliciousDemandsCollection(char* name, maliciousDemand_state * initialValue):
    ArrayStateVariable<maliciousDemand >(name, "", Struct ,20) {
    char varname[12];
    for (int i=0;i<20;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new maliciousDemand(varname, name, initialValue[i]));
    }
  }
  ~MaliciousDemandsCollection() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif
#ifndef _FR_large_collection_header_
#define _FR_large_collection_header_

typedef short FR_large_collection_state;
class FR_large_collection: public ArrayStateVariable<ExtendedPlace<short> > {
  public:
  FR_large_collection(char* name, char* fullname):
    ArrayStateVariable<ExtendedPlace<short> >(name, fullname, Short ,100) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, fqname));
    }
  }

  FR_large_collection(char* name):
    ArrayStateVariable<ExtendedPlace<short> >(name, "", Short ,100) {
    char varname[12];
    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, name));
    }
  }

  FR_large_collection(char* name, char* fullname, short & initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, fullname, Short ,100) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, fqname, initialValue));
    }
  }

  FR_large_collection(char* name, short & initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, "", Short ,100) {
    char varname[12];
    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, name, initialValue));
    }
  }

  FR_large_collection(char* name, char* fullname, short * initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, fullname, Short ,100) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, fqname, initialValue[i]));
    }
  }

  FR_large_collection(char* name, short * initialValue):
    ArrayStateVariable<ExtendedPlace<short> >(name, "", Short ,100) {
    char varname[12];
    for (int i=0;i<100;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<short>(varname, name, initialValue[i]));
    }
  }
  ~FR_large_collection() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif
#ifndef _channelState_header_
#define _channelState_header_
typedef struct channelState_state {short channel_ID; short isChannelUp; double initialPfd; double currentPfd; double commonFailurePfd; MaliciousDemandsCollection_state attacksApplied; FR_large_collection_state FR_List; };

class channelState: public StructStateVariable<channelState_state> {
  public:
  ExtendedPlace<short>* channel_ID;
  ExtendedPlace<short>* isChannelUp;
  ExtendedPlace<double>* initialPfd;
  ExtendedPlace<double>* currentPfd;
  ExtendedPlace<double>* commonFailurePfd;
  MaliciousDemandsCollection* attacksApplied;
  FR_large_collection* FR_List;
  channelState(char* name):
    StructStateVariable<channelState_state>(name, "") {
    channel_ID = new ExtendedPlace<short>("channel_ID", name);
    fields.push_back(field(Short, channel_ID));
    isChannelUp = new ExtendedPlace<short>("isChannelUp", name);
    fields.push_back(field(Short, isChannelUp));
    initialPfd = new ExtendedPlace<double>("initialPfd", name);
    fields.push_back(field(Double, initialPfd));
    currentPfd = new ExtendedPlace<double>("currentPfd", name);
    fields.push_back(field(Double, currentPfd));
    commonFailurePfd = new ExtendedPlace<double>("commonFailurePfd", name);
    fields.push_back(field(Double, commonFailurePfd));
    attacksApplied = new MaliciousDemandsCollection("attacksApplied", name);
    fields.push_back(field(Array, attacksApplied));
    FR_List = new FR_large_collection("FR_List", name);
    fields.push_back(field(Array, FR_List));
    }
  channelState(char* name, char* fullname):
    StructStateVariable<channelState_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    channel_ID = new ExtendedPlace<short>("channel_ID", varname);
    fields.push_back(field(Short, channel_ID));
    isChannelUp = new ExtendedPlace<short>("isChannelUp", varname);
    fields.push_back(field(Short, isChannelUp));
    initialPfd = new ExtendedPlace<double>("initialPfd", varname);
    fields.push_back(field(Double, initialPfd));
    currentPfd = new ExtendedPlace<double>("currentPfd", varname);
    fields.push_back(field(Double, currentPfd));
    commonFailurePfd = new ExtendedPlace<double>("commonFailurePfd", varname);
    fields.push_back(field(Double, commonFailurePfd));
    attacksApplied = new MaliciousDemandsCollection("attacksApplied", varname);
    fields.push_back(field(Array, attacksApplied));
    FR_List = new FR_large_collection("FR_List", varname);
    fields.push_back(field(Array, FR_List));
    }
  channelState(char* name, channelState_state initialValue): 
    StructStateVariable<channelState_state>(name, "") {
    channel_ID = new ExtendedPlace<short>("channel_ID", name, initialValue.channel_ID);
    fields.push_back(field(Short, channel_ID));
    isChannelUp = new ExtendedPlace<short>("isChannelUp", name, initialValue.isChannelUp);
    fields.push_back(field(Short, isChannelUp));
    initialPfd = new ExtendedPlace<double>("initialPfd", name, initialValue.initialPfd);
    fields.push_back(field(Double, initialPfd));
    currentPfd = new ExtendedPlace<double>("currentPfd", name, initialValue.currentPfd);
    fields.push_back(field(Double, currentPfd));
    commonFailurePfd = new ExtendedPlace<double>("commonFailurePfd", name, initialValue.commonFailurePfd);
    fields.push_back(field(Double, commonFailurePfd));
    attacksApplied = new MaliciousDemandsCollection("attacksApplied", name, initialValue.attacksApplied);
    fields.push_back(field(Array, attacksApplied));
    FR_List = new FR_large_collection("FR_List", name, initialValue.FR_List);
    fields.push_back(field(Array, FR_List));
    }
  channelState(char* name, char* fullname, channelState_state initialValue): 
    StructStateVariable<channelState_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    channel_ID = new ExtendedPlace<short>("channel_ID", varname, initialValue.channel_ID);
    fields.push_back(field(Short, channel_ID));
    sprintf(varname, "%s.%s", fullname, name);
    isChannelUp = new ExtendedPlace<short>("isChannelUp", varname, initialValue.isChannelUp);
    fields.push_back(field(Short, isChannelUp));
    sprintf(varname, "%s.%s", fullname, name);
    initialPfd = new ExtendedPlace<double>("initialPfd", varname, initialValue.initialPfd);
    fields.push_back(field(Double, initialPfd));
    sprintf(varname, "%s.%s", fullname, name);
    currentPfd = new ExtendedPlace<double>("currentPfd", varname, initialValue.currentPfd);
    fields.push_back(field(Double, currentPfd));
    sprintf(varname, "%s.%s", fullname, name);
    commonFailurePfd = new ExtendedPlace<double>("commonFailurePfd", varname, initialValue.commonFailurePfd);
    fields.push_back(field(Double, commonFailurePfd));
    sprintf(varname, "%s.%s", fullname, name);
    attacksApplied = new MaliciousDemandsCollection("attacksApplied", varname, initialValue.attacksApplied);
    fields.push_back(field(Array, attacksApplied));
    sprintf(varname, "%s.%s", fullname, name);
    FR_List = new FR_large_collection("FR_List", varname, initialValue.FR_List);
    fields.push_back(field(Array, FR_List));
    }
  ~channelState() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i).field;
  }
};
#endif
#ifndef _ChannelAttackList_header_
#define _ChannelAttackList_header_
typedef struct ChannelAttackList_state {short ChannelID; MaliciousDemandsCollection_state AttacksApplied; };

class ChannelAttackList: public StructStateVariable<ChannelAttackList_state> {
  public:
  ExtendedPlace<short>* ChannelID;
  MaliciousDemandsCollection* AttacksApplied;
  ChannelAttackList(char* name):
    StructStateVariable<ChannelAttackList_state>(name, "") {
    ChannelID = new ExtendedPlace<short>("ChannelID", name);
    fields.push_back(field(Short, ChannelID));
    AttacksApplied = new MaliciousDemandsCollection("AttacksApplied", name);
    fields.push_back(field(Array, AttacksApplied));
    }
  ChannelAttackList(char* name, char* fullname):
    StructStateVariable<ChannelAttackList_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ChannelID = new ExtendedPlace<short>("ChannelID", varname);
    fields.push_back(field(Short, ChannelID));
    AttacksApplied = new MaliciousDemandsCollection("AttacksApplied", varname);
    fields.push_back(field(Array, AttacksApplied));
    }
  ChannelAttackList(char* name, ChannelAttackList_state initialValue): 
    StructStateVariable<ChannelAttackList_state>(name, "") {
    ChannelID = new ExtendedPlace<short>("ChannelID", name, initialValue.ChannelID);
    fields.push_back(field(Short, ChannelID));
    AttacksApplied = new MaliciousDemandsCollection("AttacksApplied", name, initialValue.AttacksApplied);
    fields.push_back(field(Array, AttacksApplied));
    }
  ChannelAttackList(char* name, char* fullname, ChannelAttackList_state initialValue): 
    StructStateVariable<ChannelAttackList_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ChannelID = new ExtendedPlace<short>("ChannelID", varname, initialValue.ChannelID);
    fields.push_back(field(Short, ChannelID));
    sprintf(varname, "%s.%s", fullname, name);
    AttacksApplied = new MaliciousDemandsCollection("AttacksApplied", varname, initialValue.AttacksApplied);
    fields.push_back(field(Array, AttacksApplied));
    }
  ~ChannelAttackList() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i).field;
  }
};
#endif

/*********************************************************************
               software_channelSAN Submodel Definition                   
*********************************************************************/

class software_channelSAN:public SANModel{
public:

class Attack_ProcessedActivity:public Activity {
public:

  Place* successful_attack_occured;
  short* successful_attack_occured_Mobius_Mark;
  Place* attackCounter;
  short* attackCounter_Mobius_Mark;
  channelState* ChannelState;
  channelAttack* attackDetail;
  channelAttack* copyAttackDetail;
  Place* FlagSync;
  short* FlagSync_Mobius_Mark;
  Place* syncAttackSignal;
  short* syncAttackSignal_Mobius_Mark;
  Place* SystemStateUpdate;
  short* SystemStateUpdate_Mobius_Mark;
  Place* compromiseDetected;
  short* compromiseDetected_Mobius_Mark;
  FR_Array* FR_Definition;

  double* TheDistributionParameters;
  Attack_ProcessedActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // Attack_ProcessedActivityActivity

class StartCleansingActivity:public Activity {
public:

  Place* CleansingTriggered;
  short* CleansingTriggered_Mobius_Mark;
  Place* Mutex;
  short* Mutex_Mobius_Mark;
  Place* Cleansing;
  short* Cleansing_Mobius_Mark;
  channelState* ChannelState;
  Place* channel_attacked;
  short* channel_attacked_Mobius_Mark;
  Place* SystemStateUpdate;
  short* SystemStateUpdate_Mobius_Mark;

  double* TheDistributionParameters;
  StartCleansingActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // StartCleansingActivityActivity

class ChInitDoOnceActivity:public Activity {
public:

  Place* Channel_Init;
  short* Channel_Init_Mobius_Mark;
  Place* Initialised_counter;
  short* Initialised_counter_Mobius_Mark;
  channelState* ChannelState;
  Place* maxDemands;
  short* maxDemands_Mobius_Mark;
  Place* initTokens;
  short* initTokens_Mobius_Mark;

  double* TheDistributionParameters;
  ChInitDoOnceActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // ChInitDoOnceActivityActivity

class StartPatchingActivity:public Activity {
public:

  Place* PatchingTakingPlace;
  short* PatchingTakingPlace_Mobius_Mark;
  Place* Mutex;
  short* Mutex_Mobius_Mark;
  Place* compromiseDetected;
  short* compromiseDetected_Mobius_Mark;
  Place* ChannelDown;
  short* ChannelDown_Mobius_Mark;
  Place* PatchBuffer;
  short* PatchBuffer_Mobius_Mark;
  channelState* ChannelState;
  Place* channel_attacked;
  short* channel_attacked_Mobius_Mark;
  Place* SystemStateUpdate;
  short* SystemStateUpdate_Mobius_Mark;

  double* TheDistributionParameters;
  StartPatchingActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // StartPatchingActivityActivity

class bufferSyncAttackActivity:public Activity {
public:

  Place* syncroAttack_occurred;
  short* syncroAttack_occurred_Mobius_Mark;
  Place* SyncDemand;
  short* SyncDemand_Mobius_Mark;
  channelAttack* c_attackDetail;
  channelState* ChannelState;
  ChannelAttackList* SyncBuffer;

  double* TheDistributionParameters;
  bufferSyncAttackActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // bufferSyncAttackActivityActivity

class execSyncAttackActivity:public Activity {
public:

  Place* DataFromBuffer;
  short* DataFromBuffer_Mobius_Mark;
  Place* attackCounter;
  short* attackCounter_Mobius_Mark;
  channelState* ChannelState;
  channelAttack* attackDetail;
  channelAttack* copyAttackDetail;
  Place* FlagSync;
  short* FlagSync_Mobius_Mark;
  Place* syncAttackSignal;
  short* syncAttackSignal_Mobius_Mark;
  Place* SystemStateUpdate;
  short* SystemStateUpdate_Mobius_Mark;
  Place* compromiseDetected;
  short* compromiseDetected_Mobius_Mark;
  FR_Array* FR_Definition;

  double* TheDistributionParameters;
  execSyncAttackActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // execSyncAttackActivityActivity

class PatchingIntervalActivity:public Activity {
public:

  Place* PatchingEnabled;
  short* PatchingEnabled_Mobius_Mark;
  Place* PatchingTakingPlace;
  short* PatchingTakingPlace_Mobius_Mark;

  double* TheDistributionParameters;
  PatchingIntervalActivity();
  ~PatchingIntervalActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // PatchingIntervalActivityActivity

class CleansingDurationActivity:public Activity {
public:

  Place* Cleansing;
  short* Cleansing_Mobius_Mark;
  Place* CleansingCounter;
  short* CleansingCounter_Mobius_Mark;
  channelState* ChannelState;
  Place* Mutex;
  short* Mutex_Mobius_Mark;
  Place* channel_attacked;
  short* channel_attacked_Mobius_Mark;
  Place* SystemStateUpdate;
  short* SystemStateUpdate_Mobius_Mark;

  double* TheDistributionParameters;
  CleansingDurationActivity();
  ~CleansingDurationActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // CleansingDurationActivityActivity

class PatchingDurationActivity:public Activity {
public:

  Place* ChannelDown;
  short* ChannelDown_Mobius_Mark;
  Place* PatchingEnabled;
  short* PatchingEnabled_Mobius_Mark;
  Place* PatchingCounter;
  short* PatchingCounter_Mobius_Mark;
  channelState* ChannelState;
  Place* PatchBuffer;
  short* PatchBuffer_Mobius_Mark;
  Place* Mutex;
  short* Mutex_Mobius_Mark;
  Place* channel_attacked;
  short* channel_attacked_Mobius_Mark;
  Place* SystemStateUpdate;
  short* SystemStateUpdate_Mobius_Mark;
  ChannelAttackList* ChanAttackList;

  double* TheDistributionParameters;
  PatchingDurationActivity();
  ~PatchingDurationActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // PatchingDurationActivityActivity

class processSyncActivity:public Activity {
public:

  Place* SyncDemand;
  short* SyncDemand_Mobius_Mark;
  Place* DataFromBuffer;
  short* DataFromBuffer_Mobius_Mark;
  Place* FlagSync;
  short* FlagSync_Mobius_Mark;
  channelAttack* attackDetail;
  ChannelAttackList* SyncBuffer;

  double* TheDistributionParameters;
  processSyncActivity();
  ~processSyncActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // processSyncActivityActivity

  //List of user-specified place names
  Place* PatchingEnabled;
  Place* successful_attack_occured;
  Place* CleansingTriggered;
  Place* PatchingTakingPlace;
  Place* CleansingCounter;
  Place* PatchingCounter;
  Place* attackCounter;
  Place* SystemStateUpdate;
  Place* Cleansing;
  Place* Channel_Init;
  Place* Initialised_counter;
  Place* maxDemands;
  Place* initTokens;
  Place* ChannelDown;
  Place* Mutex;
  Place* channel_attacked;
  Place* syncroAttack_occurred;
  Place* SyncDemand;
  Place* DataFromBuffer;
  Place* syncAttackSignal;
  Place* FlagSync;
  Place* compromiseDetected;
  Place* PatchBuffer;
  channelAttack* attackDetail;
  channelState* ChannelState;
  FR_Array* FR_Definition;
  ChannelAttackList* ChanAttackList;
  ChannelAttackList* SyncBuffer;
  channelAttack* copyAttackDetail;
  channelAttack* c_attackDetail;

  // Create instances of all actvities
  Attack_ProcessedActivity Attack_Processed;
  StartCleansingActivity StartCleansing;
  ChInitDoOnceActivity ChInitDoOnce;
  StartPatchingActivity StartPatching;
  bufferSyncAttackActivity bufferSyncAttack;
  execSyncAttackActivity execSyncAttack;
  PatchingIntervalActivity PatchingInterval;
  CleansingDurationActivity CleansingDuration;
  PatchingDurationActivity PatchingDuration;
  processSyncActivity processSync;
  //Create instances of all groups 
  PreselectGroup ImmediateGroup;
  PostselectGroup Attack_ProcessedGroup;
  PostselectGroup StartCleansingGroup;
  PostselectGroup ChInitDoOnceGroup;
  PostselectGroup StartPatchingGroup;
  PostselectGroup bufferSyncAttackGroup;
  PostselectGroup execSyncAttackGroup;

  software_channelSAN();
  ~software_channelSAN();
  void CustomInitialization();

  void assignPlacesToActivitiesInst();
  void assignPlacesToActivitiesTimed();
}; // end software_channelSAN

#endif // software_channelSAN_H_
